import metaimageio
import numpy as np
import pytest
from test_io import file_with_suffix, SUFFIX
try:
    import imageio.v2 as imageio
except ImportError:
    import imageio


@pytest.mark.parametrize('suffix', SUFFIX)
def test_imageio(suffix):
    metaimageio.imageio()
    with file_with_suffix(suffix) as f:
        a = (100 * np.random.random_sample((2, 3, 4)))
        imageio.imwrite(f, a, format='MetaImageIO')
        b = imageio.imread(f, format='MetaImageIO')
        np.testing.assert_almost_equal(b, a)



import contextlib
import pathlib
import tempfile

import metaimageio
import numpy as np
import pytest

DTYPE = np.int8, np.uint8, np.int16, np.uint16, np.int32, np.uint32, np.int64, np.uint64, np.float32, np.float64
SUFFIX = '.mha', '.mhd'


@contextlib.contextmanager
def file_with_suffix(suffix):
    with tempfile.NamedTemporaryFile(suffix=suffix, delete=False) as f:
        filepath = pathlib.Path(f.name)
    try:
        yield filepath
    finally:
        for s in (suffix, '.raw', '.zraw'):
            fws = filepath.with_suffix(s)
            if fws.is_file():
                fws.unlink()


@pytest.mark.parametrize('dtype', DTYPE)
def test_compression(dtype):
    with file_with_suffix('.mhd') as f:
        a = (100 * np.random.random_sample((2, 3, 4))).astype(dtype)
        metaimageio.write(f, a, CompressedData=True)
        b, _ = metaimageio.read(f)
        np.testing.assert_almost_equal(b, a)


@pytest.mark.parametrize('suffix', SUFFIX)
@pytest.mark.parametrize('dtype', DTYPE)
def test_io(suffix, dtype):
    with file_with_suffix(suffix) as f:
        a = (100 * np.random.random_sample((2, 3, 4))).astype(dtype)
        metaimageio.write(f, a)
        b, _ = metaimageio.read(f)
        np.testing.assert_almost_equal(b, a)


@pytest.mark.parametrize('dtype', DTYPE)
def test_memmap(dtype):
    with file_with_suffix('.mhd') as f:
        a = (100 * np.random.random_sample((2, 3, 4))).astype(dtype)
        metaimageio.write(f, a)
        b, _ = metaimageio.read(f, memmap=True)
        np.testing.assert_almost_equal(b, a)
        del b


def test_meta():
    with file_with_suffix('.mhd') as f:
        a = np.random.random_sample((2, 3, 4))
        metaimageio.write(f, a)
        b, _ = metaimageio.read(f, slices=())
        assert b is None



import numpy as np
from imageio import formats
from imageio.core import Format

from . import reader, writer

EXTENSIONS = '.mha', '.mhd'


class MetaImageIOFormat(Format):

    def _can_read(self, request):
        return request.mode[1] in self.modes and request.extension in EXTENSIONS

    def _can_write(self, request):
        return request.mode[1] in self.modes and request.extension in EXTENSIONS

    class Reader(Format.Reader):

        def _open(self, **kwargs):
            self._filepath = self.request.get_local_filename()

        def _close(self):
            pass

        def _get_length(self):
            return np.inf

        def _get_data(self, index, **kwargs):
            if index != 0:
                raise NotImplementedError('MetaImageIO does not support non-zero indices')
            image, meta = reader.read(self._filepath, **self.request.kwargs)
            if image is None:
                image = np.array(())
            return image, meta

        def _get_meta_data(self, index):
            if index != 0:
                raise NotImplementedError('MetaImageIO does not support non-zero indices')
            _, meta = reader.read(self._filepath, slices=())
            return meta

    class Writer(Format.Writer):

        def _open(self, **kwargs):
            self._filepath = self.request.get_local_filename()

        def _close(self):
            pass

        def _append_data(self, im, meta):
            meta.pop('ElementDataFile', None)
            meta.update(self.request.kwargs)
            writer.write(self._filepath, image=im, **meta)

        def set_meta_data(self, meta):
            raise NotImplementedError('MetaImageIO does not support writing meta data')


def imageio(name='MetaImageIO'):
    if name.upper() not in formats.get_format_names():
        names = formats.get_format_names()
        formats.add_format(MetaImageIOFormat(
            name,
            'MetaImageIO',
            ' '.join(EXTENSIONS),
            'iv'))
        formats.sort(name, *names)
    return name
